@@ -18,13 +18,14 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 
 #include <simgear_config.h>
+
 #include "CanvasMap.hxx"
 #include "map/geo_node_pair.hxx"
 #include "map/projection.hxx"
 
-#include <cmath>
+#include <simgear/misc/strutils.hxx>
 
-#include <boost/algorithm/string/predicate.hpp>
+#include <cmath>
 
 #define LOG_GEO_RET(msg) \
   {\
@@ -47,6 +48,12 @@ namespace canvas
   const std::string GEO = "-geo";
   const std::string HDG = "hdg";
   const std::string Map::TYPE_NAME = "map";
+  const std::string WEB_MERCATOR = "webmercator";
+  const std::string REF_LAT = "ref-lat";
+  const std::string REF_LON = "ref-lon";
+  const std::string SCREEN_RANGE = "screen-range";
+  const std::string RANGE = "range";
+  const std::string PROJECTION = "projection";
 
   //----------------------------------------------------------------------------
   void Map::staticInit()
@@ -64,12 +71,11 @@ namespace canvas
             const SGPropertyNode_ptr& node,
             const Style& parent_style,
             ElementWeakPtr parent ):
-    Group(canvas, node, parent_style, parent),
-    // TODO make projection configurable
-    _projection(new SansonFlamsteedProjection),
-    _projection_dirty(true)
+    Group(canvas, node, parent_style, parent)
   {
     staticInit();
+
+    projectionNodeChanged(node->getChild(PROJECTION));
   }
 
   //----------------------------------------------------------------------------
@@ -79,13 +85,13 @@ namespace canvas
   }
 
   //----------------------------------------------------------------------------
-  void Map::update(double dt)
+  void Map::updateImpl(double dt)
   {
-    for( GeoNodes::iterator it = _geo_nodes.begin();
-         it != _geo_nodes.end();
-         ++it )
+    Group::updateImpl(dt);
+
+    for(auto& it: _geo_nodes)
     {
-      GeoNodePair* geo_node = it->second.get();
+      GeoNodePair* geo_node = it.second.get();
       if(    !geo_node->isComplete()
           || (!geo_node->isDirty() && !_projection_dirty) )
         continue;
@@ -107,14 +113,12 @@ namespace canvas
       geo_node->setDirty(false);
     }
     _projection_dirty = false;
-
-    Group::update(dt);
   }
 
   //----------------------------------------------------------------------------
   void Map::childAdded(SGPropertyNode* parent, SGPropertyNode* child)
   {
-    if( boost::ends_with(child->getNameString(), GEO) )
+    if( strutils::ends_with(child->getNameString(), GEO) )
       _geo_nodes[child].reset(new GeoNodePair());
     else if( parent != _node && child->getNameString() == HDG )
       _hdg_nodes.insert(child);
@@ -125,7 +129,7 @@ namespace canvas
   //----------------------------------------------------------------------------
   void Map::childRemoved(SGPropertyNode* parent, SGPropertyNode* child)
   {
-    if( boost::ends_with(child->getNameString(), GEO) )
+    if( strutils::ends_with(child->getNameString(), GEO) )
       // TODO remove from other node
       _geo_nodes.erase(child);
     else if( parent != _node && child->getName() == HDG )
@@ -147,7 +151,7 @@ namespace canvas
     {
       const std::string& name = child->getNameString();
 
-      if( boost::ends_with(name, GEO) )
+      if( strutils::ends_with(name, GEO) )
         return geoNodeChanged(child);
       else if( name == HDG )
         return hdgNodeChanged(child);
@@ -162,10 +166,10 @@ namespace canvas
     if( child->getParent() != _node )
       return Group::childChanged(child);
 
-    if(    child->getNameString() == "ref-lat"
-        || child->getNameString() == "ref-lon" )
-      _projection->setWorldPosition( _node->getDoubleValue("ref-lat"),
-                                     _node->getDoubleValue("ref-lon") );
+    if(    child->getNameString() == REF_LAT
+        || child->getNameString() == REF_LON )
+      _projection->setWorldPosition( _node->getDoubleValue(REF_LAT),
+                                     _node->getDoubleValue(REF_LON) );
     else if( child->getNameString() == HDG )
     {
       _projection->setOrientation(child->getFloatValue());
@@ -174,16 +178,43 @@ namespace canvas
                            ++it )
         hdgNodeChanged(*it);
     }
-    else if( child->getNameString() == "range" )
+    else if( child->getNameString() == RANGE )
       _projection->setRange(child->getDoubleValue());
-    else if( child->getNameString() == "screen-range" )
+    else if( child->getNameString() == SCREEN_RANGE )
       _projection->setScreenRange(child->getDoubleValue());
+    else if( child->getNameString() == PROJECTION )
+      projectionNodeChanged(child);
     else
       return Group::childChanged(child);
 
     _projection_dirty = true;
   }
 
+  //----------------------------------------------------------------------------
+  void Map::projectionNodeChanged(SGPropertyNode* child)
+  {
+    if(child && child->getStringValue() == WEB_MERCATOR)
+      _projection = std::make_shared<WebMercatorProjection>();
+    else
+      _projection = std::make_shared<SansonFlamsteedProjection>();
+
+    _projection->setWorldPosition(_node->getDoubleValue(REF_LAT),
+                                  _node->getDoubleValue(REF_LON));
+
+    // Only set existing properties to prevent using 0 instead of default values
+
+    if( auto heading = _node->getChild(HDG) )
+      _projection->setOrientation(heading->getFloatValue());
+
+    if( auto screen_range = _node->getChild(SCREEN_RANGE) )
+      _projection->setScreenRange(screen_range->getDoubleValue());
+
+    if( auto range = _node->getChild(RANGE) )
+      _projection->setRange(range->getDoubleValue());
+
+    _projection_dirty = true;
+  }
+
   //----------------------------------------------------------------------------
   void Map::geoNodeChanged(SGPropertyNode* child)
   {
