@@ -18,7 +18,7 @@
 #include <simgear/timing/timestamp.hxx>
 #include <simgear/debug/logstream.hxx>
 #include <simgear/misc/sg_dir.hxx>
-#include <simgear/misc/sgstream.hxx>
+#include <simgear/io/iostreams/sgstream.hxx>
 #include <simgear/structure/exception.hxx>
 #include <simgear/structure/callback.hxx>
 
@@ -71,7 +71,7 @@ public:
     int requestCount;
     bool getWillFail;
     bool returnCorruptData;
-    std::auto_ptr<SGCallback> accessCallback;
+    std::unique_ptr<SGCallback> accessCallback;
 
     void clearRequestCounts();
 
@@ -400,7 +400,7 @@ void waitForUpdateComplete(HTTP::Client* cl, HTTPRepository* repo)
 
 void testBasicClone(HTTP::Client* cl)
 {
-    std::auto_ptr<HTTPRepository> repo;
+    std::unique_ptr<HTTPRepository> repo;
     SGPath p(simgear::Dir::current().path());
     p.append("http_repo_basic");
     simgear::Dir pd(p);
@@ -408,7 +408,6 @@ void testBasicClone(HTTP::Client* cl)
 
     repo.reset(new HTTPRepository(p, cl));
     repo->setBaseUrl("http://localhost:2000/repo");
-    repo->setEntireRepositoryMode();
     repo->update();
 
     waitForUpdateComplete(cl, repo.get());
@@ -436,7 +435,7 @@ void testBasicClone(HTTP::Client* cl)
 
 void testModifyLocalFiles(HTTP::Client* cl)
 {
-    std::auto_ptr<HTTPRepository> repo;
+    std::unique_ptr<HTTPRepository> repo;
     SGPath p(simgear::Dir::current().path());
     p.append("http_repo_modify_local_2");
     simgear::Dir pd(p);
@@ -446,7 +445,6 @@ void testModifyLocalFiles(HTTP::Client* cl)
 
     repo.reset(new HTTPRepository(p, cl));
     repo->setBaseUrl("http://localhost:2000/repo");
-    repo->setEntireRepositoryMode();
     repo->update();
 
     waitForUpdateComplete(cl, repo.get());
@@ -478,7 +476,7 @@ void testNoChangesUpdate()
 
 void testMergeExistingFileWithoutDownload(HTTP::Client* cl)
 {
-    std::auto_ptr<HTTPRepository> repo;
+    std::unique_ptr<HTTPRepository> repo;
     SGPath p(simgear::Dir::current().path());
     p.append("http_repo_merge_existing");
     simgear::Dir pd(p);
@@ -488,7 +486,6 @@ void testMergeExistingFileWithoutDownload(HTTP::Client* cl)
 
     repo.reset(new HTTPRepository(p, cl));
     repo->setBaseUrl("http://localhost:2000/repo");
-    repo->setEntireRepositoryMode();
 
     createFile(p, "dirC/fileCB", 4); // should match
     createFile(p, "dirC/fileCC", 3); // mismatch
@@ -521,7 +518,7 @@ void testMergeExistingFileWithoutDownload(HTTP::Client* cl)
 
 void testLossOfLocalFiles(HTTP::Client* cl)
 {
-    std::auto_ptr<HTTPRepository> repo;
+    std::unique_ptr<HTTPRepository> repo;
     SGPath p(simgear::Dir::current().path());
     p.append("http_repo_lose_local");
     simgear::Dir pd(p);
@@ -531,7 +528,6 @@ void testLossOfLocalFiles(HTTP::Client* cl)
 
     repo.reset(new HTTPRepository(p, cl));
     repo->setBaseUrl("http://localhost:2000/repo");
-    repo->setEntireRepositoryMode();
     repo->update();
     waitForUpdateComplete(cl, repo.get());
     verifyFileState(p, "dirB/subdirA/fileBAA");
@@ -556,7 +552,7 @@ void testLossOfLocalFiles(HTTP::Client* cl)
 
 void testAbandonMissingFiles(HTTP::Client* cl)
 {
-    std::auto_ptr<HTTPRepository> repo;
+    std::unique_ptr<HTTPRepository> repo;
     SGPath p(simgear::Dir::current().path());
     p.append("http_repo_missing_files");
     simgear::Dir pd(p);
@@ -569,7 +565,6 @@ void testAbandonMissingFiles(HTTP::Client* cl)
 
     repo.reset(new HTTPRepository(p, cl));
     repo->setBaseUrl("http://localhost:2000/repo");
-    repo->setEntireRepositoryMode();
     repo->update();
     waitForUpdateComplete(cl, repo.get());
     if (repo->failure() != HTTPRepository::REPO_PARTIAL_UPDATE) {
@@ -581,7 +576,7 @@ void testAbandonMissingFiles(HTTP::Client* cl)
 
 void testAbandonCorruptFiles(HTTP::Client* cl)
 {
-    std::auto_ptr<HTTPRepository> repo;
+    std::unique_ptr<HTTPRepository> repo;
     SGPath p(simgear::Dir::current().path());
     p.append("http_repo_corrupt_files");
     simgear::Dir pd(p);
@@ -594,11 +589,10 @@ void testAbandonCorruptFiles(HTTP::Client* cl)
 
     repo.reset(new HTTPRepository(p, cl));
     repo->setBaseUrl("http://localhost:2000/repo");
-    repo->setEntireRepositoryMode();
-
     repo->update();
     waitForUpdateComplete(cl, repo.get());
     if (repo->failure() != HTTPRepository::REPO_ERROR_CHECKSUM) {
+        std::cerr << "Got failure state:" << repo->failure() << std::endl;
         throw sg_exception("Bad result from corrupt files test");
     }
 
@@ -611,149 +605,6 @@ void testAbandonCorruptFiles(HTTP::Client* cl)
     std::cout << "Passed test: detect corrupted download" << std::endl;
 }
 
-void testPartialUpdateBasic(HTTP::Client* cl)
-{
-    std::auto_ptr<HTTPRepository> repo;
-    SGPath p(simgear::Dir::current().path());
-    p.append("http_repo_partial_update");
-    simgear::Dir pd(p);
-    if (pd.exists()) {
-        pd.removeChildren();
-    }
-
-    global_repo->clearRequestCounts();
-    global_repo->clearFailFlags();
-    global_repo->defineFile("dirA/subdirF/fileAFA");
-    global_repo->defineFile("dirA/subdirF/fileAFB");
-    global_repo->defineFile("dirA/subdirH/fileAHA");
-    global_repo->defineFile("dirA/subdirH/fileAHB");
-
-    global_repo->defineFile("dirG/subdirA/subsubA/fileGAAB");
-
-// request subdir of A
-    repo.reset(new HTTPRepository(p, cl));
-    repo->setBaseUrl("http://localhost:2000/repo");
-    repo->addSubpath("dirA/subdirF");
-    waitForUpdateComplete(cl, repo.get());
-
-    verifyFileState(p, "dirA/subdirF/fileAFA");
-    verifyFileState(p, "dirA/subdirF/fileAFB");
-
-    verifyFileState(p, "fileA"); // files are always synced
-    verifyFileState(p, "dirA/fileAB");
-    verifyFileNotPresent(p, "dirB/subdirB/fileBBB");
-    verifyFileNotPresent(p, "dirD");
-    verifyFileNotPresent(p, "dirA/subdirH/fileAHB");
-
-    verifyRequestCount("dirA", 1);
-    verifyRequestCount("dirA/fileAA", 1);
-    verifyRequestCount("dirA/subdirF", 1);
-    verifyRequestCount("dirA/subdirF/fileAFA", 1);
-    verifyRequestCount("dirA/subdirF/fileAFB", 1);
-    verifyRequestCount("dirB", 0);
-    verifyRequestCount("dirG", 0);
-
-// now request dir B
-    repo->addSubpath("dirB");
-    waitForUpdateComplete(cl, repo.get());
-
-    verifyFileState(p, "dirA/subdirF/fileAFB");
-    verifyFileState(p, "dirB/subdirB/fileBBA");
-    verifyFileState(p, "dirB/subdirB/fileBBB");
-
-    verifyRequestCount("dirB", 1);
-    verifyRequestCount("dirB/subdirA/fileBAC", 1);
-    verifyRequestCount("dirA", 1);
-    verifyRequestCount("dirA/fileAA", 1);
-    verifyRequestCount("dirG", 0);
-
-// widen subdir to parent
-    repo->addSubpath("dirA");
-    waitForUpdateComplete(cl, repo.get());
-
-    verifyFileState(p, "dirA/subdirH/fileAHA");
-    verifyFileState(p, "dirA/subdirH/fileAHB");
-
-    verifyRequestCount("dirA", 1);
-    verifyRequestCount("dirB/subdirA/fileBAC", 1);
-    verifyRequestCount("dirA/subdirF/fileAFA", 1);
-
-// request an already fetched subdir - should be a no-op
-    repo->addSubpath("dirB/subdirB");
-    waitForUpdateComplete(cl, repo.get());
-
-    verifyRequestCount("dirB", 1);
-    verifyRequestCount("dirB/subdirB/fileBBB", 1);
-
-// add new / modify files inside
-    global_repo->defineFile("dirA/subdirF/fileAFC");
-    global_repo->defineFile("dirA/subdirF/fileAFD");
-    repo->update();
-    waitForUpdateComplete(cl, repo.get());
-
-    if (global_repo->requestCount != 2) {
-        throw sg_exception("Bad root request count");
-    }
-
-    verifyFileState(p, "dirA/subdirF/fileAFC");
-    verifyFileState(p, "dirA/subdirF/fileAFD");
-
-    std::cout << "Passed test: basic partial clone and update" << std::endl;
-}
-
-void testPartialUpdateExisting(HTTP::Client* cl)
-{
-    std::auto_ptr<HTTPRepository> repo;
-    SGPath p(simgear::Dir::current().path());
-    p.append("http_repo_partial_update_existing");
-    simgear::Dir pd(p);
-    if (pd.exists()) {
-        pd.removeChildren();
-    }
-
-    global_repo->clearRequestCounts();
-    global_repo->clearFailFlags();
-
-// full update to sync everything
-    repo.reset(new HTTPRepository(p, cl));
-    repo->setBaseUrl("http://localhost:2000/repo");
-    repo->setEntireRepositoryMode();
-    repo->update();
-    waitForUpdateComplete(cl, repo.get());
-
-// new repo for partial
-    global_repo->clearRequestCounts();
-    repo.reset(new HTTPRepository(p, cl));
-    repo->setBaseUrl("http://localhost:2000/repo");
-    repo->addSubpath("dirA/subdirF");
-    waitForUpdateComplete(cl, repo.get());
-
-    if (global_repo->requestCount != 1) {
-        throw sg_exception("Bad root request count");
-    }
-
-    verifyRequestCount("dirA", 0);
-    verifyRequestCount("dirA/fileAA", 0);
-    verifyRequestCount("dirA/subdirF", 0);
-    verifyRequestCount("dirA/subdirF/fileAFA", 0);
-    verifyRequestCount("dirA/subdirF/fileAFB", 0);
-
-// and request more dirs
-    // this is a good simulation of terrasync requesting more subdirs of
-    // an already created and in sync tree. should not generate any more
-    // network trip
-    repo->addSubpath("dirC");
-
-    verifyFileState(p, "dirC/subdirA/subsubA/fileCAAA");
-    verifyRequestCount("dirC/subdirA/subsubA/fileCAAA", 0);
-
-    if (global_repo->requestCount != 1) {
-        throw sg_exception("Bad root request count");
-    }
-
-    std::cout << "Passed test: partial update of existing" << std::endl;
-}
-
 void modifyBTree()
 {
     std::cout << "Modifying sub-tree" << std::endl;
@@ -763,53 +614,9 @@ void modifyBTree()
     global_repo->findEntry("dirB/subdirB/fileBBB")->revision++;
 }
 
-void testPartialUpdateWidenWhileInProgress(HTTP::Client* cl)
-{
-    std::auto_ptr<HTTPRepository> repo;
-    SGPath p(simgear::Dir::current().path());
-    p.append("http_repo_partial_update_widen");
-    simgear::Dir pd(p);
-    if (pd.exists()) {
-        pd.removeChildren();
-    }
-
-    global_repo->clearRequestCounts();
-    global_repo->clearFailFlags();
-
-    // full update to sync everything
-    repo.reset(new HTTPRepository(p, cl));
-    repo->setBaseUrl("http://localhost:2000/repo");
-
-    repo->addSubpath("dirA/subdirF");
-    repo->addSubpath("dirB/subdirB");
-    waitForUpdateComplete(cl, repo.get());
-
-    verifyRequestCount("dirA/subdirF", 1);
-    if (global_repo->requestCount != 1) {
-        throw sg_exception("Bad root request count");
-    }
-
-    repo->addSubpath("dirA");
-    repo->addSubpath("dirB");
-    repo->addSubpath("dirC");
-
-    waitForUpdateComplete(cl, repo.get());
-
-    // should not request the root again
-    verifyRequestCount("dirA/subdirF", 1);
-    if (global_repo->requestCount != 1) {
-        throw sg_exception("Bad root request count");
-    }
-
-    verifyFileState(p, "dirA/subdirF/fileAFA");
-    verifyFileState(p, "dirC/subdirA/subsubA/fileCAAA");
-
-    std::cout << "Passed test: partial update with widen" << std::endl;
-}
-
 void testServerModifyDuringSync(HTTP::Client* cl)
 {
-    std::auto_ptr<HTTPRepository> repo;
+    std::unique_ptr<HTTPRepository> repo;
     SGPath p(simgear::Dir::current().path());
     p.append("http_repo_server_modify_during_sync");
     simgear::Dir pd(p);
@@ -822,7 +629,6 @@ void testServerModifyDuringSync(HTTP::Client* cl)
 
     repo.reset(new HTTPRepository(p, cl));
     repo->setBaseUrl("http://localhost:2000/repo");
-    repo->setEntireRepositoryMode();
 
     global_repo->findEntry("dirA/fileAA")->accessCallback.reset(make_callback(&modifyBTree));
 
@@ -832,7 +638,7 @@ void testServerModifyDuringSync(HTTP::Client* cl)
     global_repo->findEntry("dirA/fileAA")->accessCallback.reset();
 
     if (repo->failure() != HTTPRepository::REPO_ERROR_CHECKSUM) {
-        throw sg_exception("Bad result from corrupt files test");
+        throw sg_exception("Bad result from modify during sync test");
     }
 
     std::cout << "Passed test modify server during sync" << std::endl;
@@ -841,7 +647,7 @@ void testServerModifyDuringSync(HTTP::Client* cl)
 
 void testDestroyDuringSync(HTTP::Client* cl)
 {
-    std::auto_ptr<HTTPRepository> repo;
+    std::unique_ptr<HTTPRepository> repo;
     SGPath p(simgear::Dir::current().path());
     p.append("http_repo_destory_during_sync");
     simgear::Dir pd(p);
@@ -854,7 +660,6 @@ void testDestroyDuringSync(HTTP::Client* cl)
 
     repo.reset(new HTTPRepository(p, cl));
     repo->setBaseUrl("http://localhost:2000/repo");
-    repo->setEntireRepositoryMode();
 
     repo->update();
 
@@ -869,10 +674,63 @@ void testDestroyDuringSync(HTTP::Client* cl)
     std::cout << "Passed test destory during sync" << std::endl;
 }
 
+void testCopyInstalledChildren(HTTP::Client* cl)
+{
+    std::unique_ptr<HTTPRepository> repo;
+    SGPath p(simgear::Dir::current().path());
+    p.append("http_repo_copy_installed_children");
+    simgear::Dir pd(p);
+    if (pd.exists()) {
+        pd.removeChildren();
+    }
+
+    // setup installation data
+    SGPath p2(simgear::Dir::current().path());
+    p2.append("http_repo_copy_installed_children_install");
+    simgear::Dir pd2(p2);
+    if (pd2.exists()) {
+        pd2.removeChildren();
+    } else {
+        pd2.create(0700);
+    }
+
+    // fill in 'install' tree data
+    createFile(p, "dirJ/fileJA", 2);
+    createFile(p, "dirJ/fileJB", 3);
+    createFile(p, "dirJ/fileJC", 1);
+
+    global_repo->defineFile("dirJ/fileJA", 2);
+    global_repo->defineFile("dirJ/fileJB", 3);
+    global_repo->defineFile("dirJ/fileJC", 3); // newer
+    global_repo->defineFile("dirJ/fileJD", 3); // not present in install
+
+    global_repo->clearRequestCounts();
+    global_repo->clearFailFlags();
+
+    repo.reset(new HTTPRepository(p, cl));
+    repo->setBaseUrl("http://localhost:2000/repo");
+    repo->setInstalledCopyPath(p2);
+    repo->update();
+
+    // verify correct files were downloaded, only dirs
+
+    waitForUpdateComplete(cl, repo.get());
+    verifyFileState(p, "dirJ/fileJA");
+    verifyFileState(p, "dirJ/fileJB");
+    verifyFileState(p, "dirJ/fileJC");
+    verifyFileState(p, "dirJ/fileJD");
+
+    verifyRequestCount("dirJ/fileJA", 0);
+    verifyRequestCount("dirJ/fileJB", 0);
+    verifyRequestCount("dirJ/fileJC", 1);
+    verifyRequestCount("dirJ/fileJD", 1);
+
+    std::cout << "Copy installed children" << std::endl;
+}
 
 int main(int argc, char* argv[])
 {
-  sglog().setLogLevels( SG_ALL, SG_DEBUG );
+  sglog().setLogLevels( SG_ALL, SG_INFO );
 
   HTTP::Client cl;
   cl.setMaxConnections(1);
@@ -907,13 +765,14 @@ int main(int argc, char* argv[])
     testServer.disconnectAll();
     cl.clearAllConnections();
 
-    testPartialUpdateBasic(&cl);
-    testPartialUpdateExisting(&cl);
-    testPartialUpdateWidenWhileInProgress(&cl);
-
     testServerModifyDuringSync(&cl);
-
     testDestroyDuringSync(&cl);
 
+    testServer.disconnectAll();
+    cl.clearAllConnections();
+
+    testCopyInstalledChildren(&cl);
+
+    std::cout << "all tests passed ok" << std::endl;
     return 0;
 }
