@@ -29,10 +29,7 @@
 #include <cstring>              // strerror_r() and strerror_s()
 #include <cctype>
 #include <cerrno>
-
-#if defined(HAVE_CPP11_CODECVT)
-    #include <codecvt> // new in C++11
-#endif
+#include <cassert>
 
 #include "strutils.hxx"
 
@@ -44,6 +41,8 @@
 
 #if defined(SG_WINDOWS)
 	#include <windows.h>
+    #include <codecvt>
+    #include <locale>    
 #endif
 
 using std::string;
@@ -538,6 +537,33 @@ namespace simgear {
     unsigned long long readNonNegativeInt<unsigned long long, 16>(
         const std::string& s);
 #endif
+        
+        // parse a time string ([+/-]%f[:%f[:%f]]) into hours
+        double readTime(const string& time_in)
+        {
+            if (time_in.empty()) {
+                return 0.0;
+            }
+            
+            const bool negativeSign = time_in.front() == '-';
+            const string_list pieces = split(time_in, ":");
+            if (pieces.size() > 3) {
+                throw sg_format_exception("Unable to parse time string, too many pieces", time_in);
+            }
+            
+            const int hours = std::abs(to_int(pieces.front()));
+            int minutes = 0, seconds = 0;
+            if (pieces.size() > 1) {
+                minutes = to_int(pieces.at(1));
+                if (pieces.size() > 2) {
+                    seconds = to_int(pieces.at(2));
+                }
+            }
+        
+            double result = hours + (minutes / 60.0) + (seconds / 3600.0);
+            return negativeSign ? -result : result;
+        }
+
 
     int compare_versions(const string& v1, const string& v2, int maxComponents)
     {
@@ -629,25 +655,85 @@ static std::string convertWStringToMultiByte(DWORD encoding, const std::wstring&
 
 std::wstring convertUtf8ToWString(const std::string& a)
 {
-#ifdef SG_WINDOWS
-    return convertMultiByteToWString(CP_UTF8, a);
-#elif defined(HAVE_CPP11_CODECVT)
+#if defined(SG_WINDOWS)
     std::wstring_convert<std::codecvt_utf8<wchar_t>, wchar_t> ucs2conv;
     return ucs2conv.from_bytes(a);
 #else
-    return std::wstring();
+    assert(sizeof(wchar_t) == 4);
+    std::wstring result;
+    int expectedContinuationCount = 0;
+    wchar_t wc = 0;
+    
+    for (uint8_t utf8CodePoint : a) {
+        // ASCII 7-bit range
+        if (utf8CodePoint <= 0x7f) {
+            if (expectedContinuationCount != 0) {
+                throw sg_format_exception();
+            }
+            
+            result.push_back(static_cast<wchar_t>(utf8CodePoint));
+        } else if (expectedContinuationCount > 0) {
+            if ((utf8CodePoint & 0xC0) != 0x80) {
+                throw sg_format_exception();
+            }
+            
+            wc = (wc << 6) | (utf8CodePoint & 0x3F);
+            if (--expectedContinuationCount == 0) {
+                result.push_back(wc);
+            }
+        } else {
+            if ((utf8CodePoint & 0xE0) == 0xC0) {
+                expectedContinuationCount = 1;
+                wc = utf8CodePoint & 0x1f;
+            } else if ((utf8CodePoint & 0xF0) == 0xE0) {
+                expectedContinuationCount = 2;
+                wc = utf8CodePoint & 0x0f;
+            } else if ((utf8CodePoint & 0xF8) == 0xF0) {
+                expectedContinuationCount = 3;
+                wc =utf8CodePoint & 0x07;
+            } else {
+                // illegal UTF-8 encoding
+                throw sg_format_exception();
+            }
+        }
+    } // of UTF-8 code point iteration
+    
+    return result;
+    
 #endif
+
 }
 
 std::string convertWStringToUtf8(const std::wstring& w)
 {
-#ifdef SG_WINDOWS
-	return convertWStringToMultiByte(CP_UTF8, w);
-#elif defined(HAVE_CPP11_CODECVT)
+#if defined(SG_WINDOWS)
     std::wstring_convert<std::codecvt_utf8<wchar_t>, wchar_t> ucs2conv;
     return ucs2conv.to_bytes(w);
 #else
-    return std::string();
+    assert(sizeof(wchar_t) == 4);
+    std::string result;
+    
+    for (wchar_t cp : w) {
+        if (cp <= 0x7f) {
+            result.push_back(static_cast<uint8_t>(cp));
+        } else if (cp <= 0x07ff) {
+            result.push_back(0xC0 | ((cp >> 6) & 0x1f));
+            result.push_back(0x80 | (cp & 0x3f));
+        } else if (cp <= 0xffff) {
+            result.push_back(0xE0 | ((cp >> 12) & 0x0f));
+            result.push_back(0x80 | ((cp >> 6) & 0x3f));
+            result.push_back(0x80 | (cp & 0x3f));
+        } else if (cp < 0x10ffff) {
+            result.push_back(0xF0 | ((cp >> 18) & 0x07));
+            result.push_back(0x80 | ((cp >> 12) & 0x3f));
+            result.push_back(0x80 | ((cp >> 6) & 0x3f));
+            result.push_back(0x80 | (cp & 0x3f));
+        } else {
+            throw sg_format_exception();
+        }
+    }
+    
+    return result;
 #endif
 }
 
