@@ -19,6 +19,7 @@
 #include <simgear/compiler.h>
 #include <simgear/misc/strutils.hxx>
 #include <simgear/structure/exception.hxx>
+#include <simgear/constants.h>
 
 using std::string;
 using std::vector;
@@ -581,6 +582,43 @@ void test_error_string()
   SG_CHECK_GT(strutils::error_string(saved_errno).size(), 0);
 }
 
+void test_readTime()
+{
+    SG_CHECK_EQUAL_EP(strutils::readTime(""), 0.0);
+
+    SG_CHECK_EQUAL_EP(strutils::readTime("11"), 11.0);
+    SG_CHECK_EQUAL_EP(strutils::readTime("+11"), 11.0);
+    SG_CHECK_EQUAL_EP(strutils::readTime("-11"), -11.0);
+    
+    SG_CHECK_EQUAL_EP(strutils::readTime("11:30"), 11.5);
+    SG_CHECK_EQUAL_EP(strutils::readTime("+11:15"), 11.25);
+    SG_CHECK_EQUAL_EP(strutils::readTime("-11:45"), -11.75);
+    
+    const double seconds = 1 / 3600.0;
+    SG_CHECK_EQUAL_EP(strutils::readTime("11:30:00"), 11.5);
+    SG_CHECK_EQUAL_EP(strutils::readTime("+11:15:05"), 11.25 + 5 * seconds);
+    SG_CHECK_EQUAL_EP(strutils::readTime("-11:45:15"), -(11.75 + 15 * seconds));
+
+    SG_CHECK_EQUAL_EP(strutils::readTime("0:0:0"), 0);
+    
+    SG_CHECK_EQUAL_EP(strutils::readTime("0:0:28"), 28 * seconds);
+    SG_CHECK_EQUAL_EP(strutils::readTime("-0:0:28"), -28 * seconds);
+}
+
+void test_utf8Convert()
+{
+    // F, smiley emoticon, Maths summation symbol, section sign
+    std::wstring a(L"\u0046\U0001F600\u2211\u00A7");
+    
+    
+    std::string utf8A = strutils::convertWStringToUtf8(a);
+    SG_VERIFY(utf8A == std::string("F\xF0\x9F\x98\x80\xE2\x88\x91\xC2\xA7"));
+    
+    
+    std::wstring aRoundTrip = strutils::convertUtf8ToWString(utf8A);
+    SG_VERIFY(a == aRoundTrip);
+}
+
 int main(int argc, char* argv[])
 {
     test_strip();
@@ -599,6 +637,8 @@ int main(int argc, char* argv[])
     test_md5_hex();
     test_error_string();
     test_propPathMatch();
-
+    test_readTime();
+    test_utf8Convert();
+    
     return EXIT_SUCCESS;
 }
