@@ -15,6 +15,7 @@
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
 
+#include <simgear_config.h>
 #include <simgear/package/Install.hxx>
 
 #include <boost/foreach.hpp>
@@ -33,7 +34,7 @@
 #include <simgear/io/HTTPClient.hxx>
 #include <simgear/misc/sg_dir.hxx>
 #include <simgear/misc/strutils.hxx>
-#include <simgear/misc/sgstream.hxx>
+#include <simgear/io/iostreams/sgstream.hxx>
 
 extern "C" {
     void fill_memory_filefunc (zlib_filefunc_def*);
@@ -152,6 +153,11 @@ protected:
             return;
         }
 
+		// disable caching on the owner's path, otherwise the upcoming
+		// delete & rename confuse everything
+		m_owner->m_path.set_cached(false);
+		m_extractPath.set_cached(false);
+
         if (m_owner->path().exists()) {
             Dir destDir(m_owner->path());
             destDir.remove(true /* recursive */);
@@ -450,33 +456,6 @@ void Install::cancelDownload()
     m_package->catalog()->root()->cancelDownload(this);
 }
 
-struct PathAppender
-{
-    PathAppender(const SGPath& p) : m_path(p) {}
-
-    SGPath operator()(const std::string& s) const
-    {
-        SGPath p(m_path);
-        p.append(s);
-        return p;
-    }
-
-    SGPath m_path;
-};
-
-PathList Install::thumbnailPaths() const
-{
-    const string_list& thumbs(m_package->thumbnails());
-    PathList result;
-    if (thumbs.empty())
-        return result;
-
-    std::transform(thumbs.begin(), thumbs.end(),
-                   std::back_inserter(result),
-                   PathAppender(m_path));
-    return result;
-}
-
 SGPath Install::primarySetPath() const
 {
     SGPath setPath(m_path);
