@@ -1,4 +1,4 @@
-// ModelRegistry.hxx -- interface to the OSG model registry
+// ModelRegistry.cxx -- interface to the OSG model registry
 //
 // Copyright (C) 2005-2007 Mathias Froehlich
 // Copyright (C) 2007  Tim Moore <timoore@redhat.com>
@@ -35,6 +35,7 @@
 #include <osg/Switch>
 #include <osg/Material>
 #include <osg/MatrixTransform>
+#include <osg/Version>
 #include <osgDB/Archive>
 #include <osgDB/FileNameUtils>
 #include <osgDB/FileUtils>
@@ -415,7 +416,12 @@ public:
   {
     // XXX I understand why we want this, but this seems like a weird
     // place to set this option.
+#if OSG_VERSION_LESS_THAN(3,5,4)
+      //RJH - see OSG 11ddd53eb46d15903d036b594bfa3826d9e89393 -
+      //      Removed redundent Referenced::s/getThreadSafeReferenceCounting() and associated static and env vars
+      //      as there are now inapprorpiate and no longer supported
     Referenced::setThreadSafeReferenceCounting(true);
+#endif
 
     Registry* registry = Registry::instance();
     Options* options = new Options;
@@ -457,8 +463,13 @@ struct ACOptimizePolicy : public OptimizeModelPolicy {
         }
         const SGReaderWriterOptions* sgopt
             = dynamic_cast<const SGReaderWriterOptions*>(opt);
-        if (sgopt && sgopt->getInstantiateEffects())
+
+        if (sgopt && sgopt->getInstantiateMaterialEffects()) {
+            optimized = instantiateMaterialEffects(optimized.get(), sgopt);
+        } else if (sgopt && sgopt->getInstantiateEffects()) {
             optimized = instantiateEffects(optimized.get(), sgopt);
+        }
+
         return optimized.release();
     }
 };
