@@ -416,15 +416,11 @@ private:
 } // namespace
 
 
-SGMaterialAnimation::SGMaterialAnimation(const SGPropertyNode* configNode,
-                                         SGPropertyNode* modelRoot,
-                                         const osgDB::Options*
-                                         options, const string &path) :
-  SGAnimation(configNode, modelRoot),
-  texturePathList(options->getDatabasePathList())
+SGMaterialAnimation::SGMaterialAnimation(simgear::SGTransientModelData &modelData) :
+    SGAnimation(modelData), texturePathList(modelData.getOptions()->getDatabasePathList())
 {
-  if (configNode->hasChild("global"))
-    SG_LOG(SG_IO, SG_ALERT, path <<
+  if (modelData.getConfigNode()->hasChild("global"))
+    SG_LOG(SG_IO, SG_ALERT, modelData.getPath() <<
            ": Use of <global> in material animation is "
            "no longer supported.");
 }
@@ -450,7 +446,8 @@ SGMaterialAnimation::createAnimationGroup(osg::Group& parent)
         stateSet->setTextureAttribute(0, texture2D,
 				      osg::StateAttribute::OVERRIDE);
         stateSet->setTextureMode(0, GL_TEXTURE_2D, osg::StateAttribute::ON);
-        if (texture2D->getImage()->isImageTranslucent()) {
+		osg::ref_ptr<osg::Image> textureImage = texture2D->getImage();
+        if (textureImage && textureImage->isImageTranslucent()) {
           stateSet->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);
           stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);
         }
